package biotextEngine.util;
import java.sql.*;
import java.util.*;

public class BioTextDBConnection {

    /* Database connection specific parameters, defined in the files
     * config.properties
     */
    private ResourceBundle rb = ResourceBundle.getBundle("config");

    private String driverName = rb.getString("driverName");
    private String host = rb.getString("host");
    private String port = rb.getString("port");
    private String dbname = rb.getString("dbname");
    private String user = rb.getString("user");
    private String passwd = rb.getString("passwd");
    private String schema = rb.getString("schema");
    private String urlprefix = rb.getString("urlprefix");

    private Connection dbConnection;

    public BioTextDBConnection() throws Exception{
        connect();
        setSchema(schema);
    }

    public BioTextDBConnection(String schema) throws Exception {
        connect();
        setSchema(schema);
    }

    public void connect() throws Exception {

        try {
            Class.forName(driverName);
            System.err.println("Opening db connection");
            String url = urlprefix + host + ":" + port + "/" + dbname;
            dbConnection = DriverManager.getConnection(url, user, passwd);

            
        }
        catch (ClassNotFoundException ex) {
            System.err.println("Cannot find the database driver classes.");
            throw new Exception (ex.getMessage());
        }
        catch (SQLException ex) {
            System.err.println("Cannot connect to this database.");
            throw new Exception (ex.getMessage());
        }
    }

    public void setSchema(String schema) throws Exception {

        if (schema != null) {
            try {
                Statement stmt = dbConnection.createStatement();
                stmt.executeUpdate("set schema " + schema);
                stmt.close();
                this.schema = schema;
            }
            catch (SQLException e) {
                throw new Exception ("Cannot set the current schema to " + schema);
            }
        }
    }


    public Connection getConnection() {

        return dbConnection;
    }

    /*public void finalize() {
        try {
            dbConnection.close();
        }
        catch (SQLException e) {
            System.err.println("Unable to close the connection to the Database");
        }
    }*/
}
