/**
 * The main handler for the medline documents. Handles the top level node
 * <MedlineCitation>
 *
 * @author Ariel Schwartz
 * @author Gaurav Bhalotia
 *      
 */
package biotextEngine.xmlparsers.medline;
import biotextEngine.xmlparsers.NodeHandler;
import biotextEngine.xmlparsers.GenericXMLParser;
import java.sql.*;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
 
public class MedlineCitation extends NodeHandler{

    String pmid;
    private static PreparedStatement pstmt_delete = null;

    private static boolean isIncremental = true;

    /* Default constructor
     *
     * @param xmlFileName The name of the file that is being parsed, to be stored in the
     *        record for medline_citation
     */
    public MedlineCitation(String xmlFileName) throws Exception{

        ignoreDuplicateKeyError = true;

        /* The table in which the parsed entries are to be entered from this node */
        tableName = "medline_citation";

        /* The node name being handled by this class */
	xmlNodeName = "MedlineCitation";

        /* The various column names in the table */
	String[] columnNameDef = {"pmid", 
				  "date_created",  
				  "date_completed",
				  "date_revised",
				  "issn", 
				  "volume",
				  "issue",
				  "pub_date_year",
				  "pub_date_month",
				  "pub_date_day",
				  "pub_date_season",
				  "medline_date",
				  "journal_print_yn",
				  "coden",
				  "journal_title",
				  "iso_abbreviation",
				  "article_title",
				  "start_page",
				  "end_page",
				  "medline_pgn",
				  "abstract_text",
				  "copyright_info",
				  "article_affiliation",
				  "article_author_list_comp_yn",
                                  "data_bank_list_comp_yn",
                                  "grantlist_complete_yn",
				  "vernacular_title",
				  "date_of_electronic_publication",
				  "elec_pub_official_date_yn",
				  "country",
				  "medline_ta",
				  "nlm_unique_id",
				  "xml_file_name",
				  "number_of_references",
				  "keyword_list_owner",
				  "citation_owner",
				  "citation_status"
	};

	columnName = columnNameDef;
	
        /* The corresponding XML tags, The tags names starts from the current xmlnode */
	String[] xmlElementNameDef = {"MedlineCitation.PMID",
				      "MedlineCitation.DateCreated",
				      "MedlineCitation.DateCompleted",
				      "MedlineCitation.DateRevised",	
				      "MedlineCitation.Article.Journal.ISSN", 
				      "MedlineCitation.Article.Journal.JournalIssue.Volume",
				      "MedlineCitation.Article.Journal.JournalIssue.Issue",
				      "MedlineCitation.Article.Journal.JournalIssue.PubDate.Year",
				      "MedlineCitation.Article.Journal.JournalIssue.PubDate.Month",
				      "MedlineCitation.Article.Journal.JournalIssue.PubDate.Day",
				      "MedlineCitation.Article.Journal.JournalIssue.PubDate.Season",
				      "MedlineCitation.Article.Journal.JournalIssue.PubDate.MedlineDate",
				      "MedlineCitation.Article.Journal.JournalIssue.PrintYN",
				      "MedlineCitation.Article.Journal.Coden",
				      "MedlineCitation.Article.Journal.Title",
				      "MedlineCitation.Article.Journal.ISOAbbreviation",
				      "MedlineCitation.Article.ArticleTitle",
				      "MedlineCitation.Article.Pagination.StartPage",
				      "MedlineCitation.Article.Pagination.EndPage",
				      "MedlineCitation.Article.Pagination.MedlinePgn",
				      "MedlineCitation.Article.Abstract.AbstractText",
				      "MedlineCitation.Article.Abstract.CopyrightInformation",
				      "MedlineCitation.Article.Affiliation",
				      "MedlineCitation.Article.AuthorList.CompleteYN",
                                      "MedlineCitation.Article.DataBankList.CompleteYN",
                                      "MedlineCitation.Article.GrantList.CompleteYN",
				      "MedlineCitation.Article.VernacularTitle",
				      "MedlineCitation.Article.ElectronicPubDate",
				      "MedlineCitation.Article.ElectronicPubDate.OfficialDateYN",     
				      "MedlineCitation.MedlineJournalInfo.Country",
				      "MedlineCitation.MedlineJournalInfo.MedlineTA",
				      "MedlineCitation.MedlineJournalInfo.NlmUniqueID",
				      "XmlFileName",
				      "MedlineCitation.NumberOfReferences",
				      "MedlineCitation.KeywordList.Owner", 
				      "MedlineCitation.Owner", 
				      "MedlineCitation.Status" 
	};

        xmlElementName = xmlElementNameDef;

        /* The SQL types for the various columns above */
	int columnTypeDef[] = {Types.INTEGER,
			       Types.DATE,
			       Types.DATE,
			       Types.DATE,
			       Types.CHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.CHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.CLOB,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.CHAR,
			       Types.CHAR,
			       Types.CHAR,
			       Types.VARCHAR,
			       Types.DATE,
			       Types.CHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.INTEGER,
			       Types.VARCHAR,
			       Types.VARCHAR,
			       Types.VARCHAR
	};

        columnType = columnTypeDef;
        initialize();

	/* Add any data that does not come through XML to the hashtable */
        putColumnValue("XmlFileName", xmlFileName);
    }

    /**
     * The method to handle the event when a new element is found, this is overwriting
     * the method defined in the super class NodeHandler.java
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) 
	throws SAXException {

        NodeHandler handler;

        String descriptorName = null;
        String majorTopicYN = null;

        /* Take decisions based on the element found, if it needs to be handled
         * by a child handler then instantiate an object for the same and set the handler
         * else call the handler from the super class
         */
	try{
	    if (currentElement != null) {
                if (pmid == null)
                    pmid = getColumnValue("MedlineCitation.PMID");

                if (currentElement.equals("AuthorList") && qName.equals("Author")){
                    handler = new Author(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("AccessionNumberList") && qName.equals("AccessionNumber")){
		    String dataBankName = getColumnValue("MedlineCitation.Article.DataBankList.DataBank.DataBankName");
                    handler = new DataBank(pmid,dataBankName);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("ChemicalList") && qName.equals("Chemical")){
                    handler = new Chemical(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("GeneSymbolList") && qName.equals("GeneSymbol")){
                    handler = new GeneSymbol(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("KeywordList") && qName.equals("Keyword")){
                    handler = new Keyword(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("PublicationTypeList") && qName.equals("PublicationType")){
                    handler = new ArticlePublicationType(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("GrantList") && qName.equals("Grant")){
                    handler = new Grant(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("MeshHeadingList") && qName.equals("MeshHeading")) {
                    handler = new MeshHeading(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("CommentsCorrections")) {
                    handler = new CommentsCorrections(pmid, qName);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (qName.equals("CitationSubset")) {
                    handler = new CitationSubset(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (qName.equals("Language")) {
                    handler = new ArticleLanguage(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("PersonalNameSubjectList") && qName.equals("PersonalNameSubject")){
                    handler = new PersonalNameSubject(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (qName.equals("OtherID")) {
                    handler = new OtherID(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (qName.equals("OtherAbstract")) {
                    handler = new OtherAbstract(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (qName.equals("SpaceFlightMission")) {
                    handler = new SpaceFlightMission(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (currentElement.equals("InvestigatorList") && qName.equals("Investigator")){
                    handler = new Investigator(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else if (qName.equals("GeneralNote")) {
                    handler = new GeneralNote(pmid);
                    setContentHandler(handler, namespaceURI, localName, qName, atts);
                }
                else {
                    super.startElement(namespaceURI, localName, qName, atts);
                }
	    } else {
		super.startElement(namespaceURI, localName, qName, atts);
	    }
	}catch (Exception e) {
	    e.printStackTrace();
	    throw new SAXException("Problem creating Child. PMID: " + pmid + " for element " + qName);
	}
    }
    
    /* Extends the endElement method from the super class NodeHandler
     * checks if a medline citation has ended, in which case flushes the
     * values found including all the childhandler to the database
     */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
	super.endElement(namespaceURI, localName, qName);
	if (qName.equals(xmlNodeName)){
	    try {
		updateDB();
	    } catch (Exception e) {
		e.printStackTrace();
		throw new SAXException("problem updating the database");
	    }
	}
    }


    /**
     * Handles SQLException. Should be overloaded by inheriting classes to handle special cases
     * @returns true if the exception has been handled, false otherwise 
     */
    protected boolean handleSQLException(SQLException e) {
        if (ignoreDuplicateKeyError && e.getErrorCode() == DB2_DUPLICATE_ERROR) {

            if (isIncremental) {
                try {

                    if (pstmt_delete == null) {
                        pstmt_delete = GenericXMLParser.getDbConnection().prepareStatement("DELETE FROM medline_citation where pmid = ?");
                    }

                    pstmt_delete.setInt(1, Integer.parseInt(pmid));

                    pstmt_delete.executeUpdate();

                    /* Execute the insert again. Note this could cause problems in multithreaded implementation */
                    pstmt.executeUpdate();
                    return true;
                }
                catch (SQLException e1) {
                    /* Doesn't work again so give up and report */
                    MedlineParser.eCount++;
                    if (MedlineParser.eCount % 500 == 0) {
                        System.out.println("Total " + MedlineParser.eCount + " Values not inserted");
                    }
                    updateChildren = false;
                    return true;
                }

            }
            else {

                MedlineParser.eCount++;
                if (MedlineParser.eCount % 500 == 0) {
                    System.out.println("Total " + MedlineParser.eCount + " Values not inserted");
                }
                updateChildren = false;
                return true;
                /* Don't do anything, the tuple for this primary key has already been inserted */
            }
        }
        else {
            System.err.println("ERROR CODE == " + e.getErrorCode());
            return false;
        }
    }

}
